//  $Id: world.cpp,v 1.1 2006/05/02 23:56:57 ondra Exp $
// 
//  SuperTux
//  Copyright (C) 2000 Bill Kendrick <bill@newbreedsoftware.com>
//  Copyright (C) 2004 Tobias Glaesser <tobi.web@gmx.de>
//  Copyright (C) 2004 Ingo Ruhnke <grumbel@gmx.de>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.

#include <iostream>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "globals.h"
#include "scene.h"
#include "screen.h"
#include "defines.h"
#include "world.h"
#include "level.h"
#include "tile.h"
#include "resources.h"

extern float myScale;

Surface* img_distro[4];

World* World::current_ = 0;

World::World(const std::string& filename)
{
  // FIXME: Move this to action and draw and everywhere else where the
  // world calls child functions
  current_ = this;

  level = new Level(filename);
  tux.init();

  set_defaults();

  get_level()->load_gfx();
  activate_bad_guys();
  activate_particle_systems();
  get_level()->load_song();

  apply_bonuses();

  scrolling_timer.init(true);
}

World::World(const std::string& subset, int level_nr)
{
  // FIXME: Move this to action and draw and everywhere else where the
  // world calls child functions
  current_ = this;

  level = new Level(subset, level_nr);
  tux.init();

  set_defaults();

  get_level()->load_gfx();
  activate_bad_guys();
  activate_particle_systems();
  get_level()->load_song();

  apply_bonuses();

  scrolling_timer.init(true);
}

void
World::apply_bonuses()
{
  // Apply bonuses from former levels
  switch (player_status.bonus)
    {
    case PlayerStatus::NO_BONUS:
      break;

    case PlayerStatus::FLOWER_BONUS:
      tux.got_coffee = true;
      // fall through

    case PlayerStatus::GROWUP_BONUS:
      // FIXME: Move this to Player class
      tux.size = BIG;
      tux.base.height = 64/myScale;
      tux.base.y -= 32/myScale;
      break;
    }
}

World::~World()
{
  for (BadGuys::iterator i = bad_guys.begin(); i != bad_guys.end(); ++i)
    delete *i;

  for (ParticleSystems::iterator i1 = particle_systems.begin();
          i1 != particle_systems.end(); ++i1)
    delete *i1;

  for (std::vector<BouncyDistro*>::iterator i2 = bouncy_distros.begin();
       i2 != bouncy_distros.end(); ++i2)
    delete *i2;
  
  for (std::vector<BrokenBrick*>::iterator i3 = broken_bricks.begin();
       i3 != broken_bricks.end(); ++i3)
    delete *i3;
  
  for (std::vector<BouncyBrick*>::iterator i4 = bouncy_bricks.begin();
       i4 != bouncy_bricks.end(); ++i4)
    delete *i4;

  for (std::vector<FloatingScore*>::iterator i5 = floating_scores.begin();
       i5 != floating_scores.end(); ++i5)
    delete *i5;
  
  delete level;
}

void
World::set_defaults()
{
  // Set defaults: 
  scroll_x = 0;

  player_status.score_multiplier = 1;

  counting_distros = false;
  distro_counter = 0;

  /* set current song/music */
  currentmusic = LEVEL_MUSIC;
}

void
World::activate_bad_guys()
{
  for (std::vector<BadGuyData>::iterator i = level->badguy_data.begin();
       i != level->badguy_data.end();
       ++i)
    {
//      add_bad_guy((float)(i->x/(float)myScale), (float)(i->y/(float)myScale), i->kind, i->stay_on_platform);
// ONDRA HACK - snizeni y souradnice o 10px tak, aby se nektri BG nezasekli
		add_bad_guy((float)(i->x), (float)(i->y - 16), i->kind, i->stay_on_platform);
    }
}

void
World::activate_particle_systems()
{
  if (level->particle_system == "clouds")
    {
      particle_systems.push_back(new CloudParticleSystem);
    }
  else if (level->particle_system == "snow")
    {
      particle_systems.push_back(new SnowParticleSystem);
    }
  else if (level->particle_system != "")
    {
      st_abort("unknown particle system specified in level", "");
    }
}

void
World::draw()
{
  int y,x;

  /* Draw the real background */
  if(level->img_bkgd)
    {
      int s = (int)((float)scroll_x * ((float)level->bkgd_speed/100.0f)) % screen->w;
      level->img_bkgd->draw_part(s, 0,0,0,level->img_bkgd->w - s, level->img_bkgd->h);
      level->img_bkgd->draw_part(0, 0,screen->w - s ,0,s,level->img_bkgd->h);
    }
  else
    {
      drawgradient(level->bkgd_top, level->bkgd_bottom);
    }
    
  /* Draw particle systems (background) */
  std::vector<ParticleSystem*>::iterator p;
  for(p = particle_systems.begin(); p != particle_systems.end(); ++p)
    {
      (*p)->draw(scroll_x, 0, 0);
    }

  /* Draw background: */
  for (y = 0; y < 15; ++y)
    {
      for (x = 0; x < 21; ++x)
        {
          Tile::draw(32*x/myScale - fmodf(scroll_x, 32/myScale), y * 32/myScale,
                     level->bg_tiles[(int)y][(int)x + (int)(scroll_x / (32/myScale))]);
//		  BOUNCING_BOX(32*x/myScale - fmodf(scroll_x, 32/myScale), y * 32/myScale,\
			  32/myScale, 1, 0, 0, 255, 255);
        }
    }

  /* Draw interactive tiles: */
  for (y = 0; y < 15; ++y)
    {
      for (x = 0; x < 21; ++x)
        {
          Tile::draw(32*x/myScale - fmodf(scroll_x, 32/myScale), y * 32/myScale,
                     level->ia_tiles[(int)y][(int)x + (int)(scroll_x / (32/myScale))]);
//		  BOUNCING_BOX(32*x/myScale - fmodf(scroll_x, 32/myScale), y * 32/myScale,\
			  32/myScale, 1, 0, 255, 0, 255);
        }
    }

  /* (Bouncy bricks): */
  unsigned int i = 0;
  for (i = 0; i < bouncy_bricks.size(); ++i)
    bouncy_bricks[i]->draw();

  for (BadGuys::iterator it = bad_guys.begin(); it != bad_guys.end(); ++it)
    (*it)->draw();

  tux.draw();

  for (i = 0; i < bullets.size(); ++i)
    bullets[i].draw();

  for (i = 0; i < floating_scores.size(); ++i)
    floating_scores[i]->draw();

  for (i = 0; i < upgrades.size(); ++i)
    upgrades[i].draw();

  for (i = 0; i < bouncy_distros.size(); ++i)
    bouncy_distros[i]->draw();

  for (i = 0; i < broken_bricks.size(); ++i)
    broken_bricks[i]->draw();

  /* Draw foreground: */
  for (y = 0; y < 15; ++y)
    {
      for (x = 0; x < 21; ++x)
        {
          Tile::draw(32*x/myScale - fmodf(scroll_x, 32/myScale), y * 32/myScale,
                     level->fg_tiles[(int)y][(int)x + (int)(scroll_x / (32/myScale))]);
//		  BOUNCING_BOX(32*x/myScale - fmodf(scroll_x, 32/myScale), y * 32/myScale,\
			  32/myScale, 1, 255, 0, 0, 255);
        }
    }

  /* Draw particle systems (foreground) */
  for(p = particle_systems.begin(); p != particle_systems.end(); ++p)
    {
      (*p)->draw(scroll_x, 0, 1);
    }


  int vyska = (screen->h - 480/myScale)/(32/myScale) + 1 + 15;
  fillrect(0,480/myScale,screen->w,screen->h,0,0,0,255);	

  for (y = 15; y < vyska; ++y)
    {
      for (x = 0; x < 21; ++x)
        {
          /*Tile::draw(32*x/myScale - fmodf(scroll_x, 32/myScale), y * 32/myScale,
                     level->fg_tiles[(int)14][(int)x + (int)(scroll_x / (32/myScale))]);*/
		  Tile::draw(32*x/myScale - fmodf(scroll_x, 32/myScale), y * 32/myScale,
                     level->bg_tiles[(int)14][(int)x + (int)(scroll_x / (32/myScale))]);
        }
    }

  

  soft_button_H = max( screen->h - 480/myScale, screen->h / 3) ;
  soft_button_W = screen->w / 3;
//  soft_button_W = soft_button_H;
//  soft_button_X = screen->w - 	soft_button_W;
  int alpha = 128;
  if(screen->h > screen->w)
  {
	  soft_button_H = max( screen->h - 480/myScale, 60 / myScale) ;
	  soft_button_X = 0;
	  soft_button_Y = screen->h - soft_button_H;
	  soft_button1_H = soft_button_H;
	  soft_button1_W = soft_button_W;
	  soft_button1_X = soft_button_X + soft_button_W;	
	  soft_button1_Y = soft_button_Y;
	  soft_button2_H = soft_button_H;
	  soft_button2_W = soft_button_W;
	  soft_button2_X = soft_button1_X + soft_button_W;	
	  soft_button2_Y = soft_button_Y;
	  alpha = 255;
  }
  else
  {
	  soft_button_X = screen->w - soft_button_W;
	  soft_button_Y = soft_button_H;
	  soft_button2_H = soft_button_H;
	  soft_button2_W = soft_button_W;
	  soft_button2_X = soft_button_X;
	  soft_button2_Y = soft_button_Y - soft_button_H;
	  soft_button1_H = soft_button_H;
	  soft_button1_W = soft_button_W;
	  soft_button1_X = soft_button2_X - soft_button2_W;
	  soft_button1_Y = soft_button2_Y;
  }

  fillrect(soft_button_X,soft_button_Y,soft_button_W,soft_button_H,128,180,128,alpha);
  fillrect(soft_button1_X,soft_button1_Y,soft_button1_W,soft_button1_H,128,128,180,alpha);
  fillrect(soft_button2_X,soft_button2_Y,soft_button2_W,soft_button2_H,180,128,128,alpha);
  white_text->draw_align("FIRE",
                   soft_button_X + soft_button_W / 2, soft_button_Y + soft_button_H / 2,
                   A_HMIDDLE, A_VMIDDLE, 1);
  white_text->draw_align("FIRE+JUMP",
                   soft_button1_X + soft_button1_W / 2, soft_button1_Y + soft_button1_H / 2,
                   A_HMIDDLE, A_VMIDDLE, 1);
  white_text->draw_align("JUMP",
                   soft_button2_X + soft_button2_W / 2, soft_button2_Y + soft_button2_H / 2,
                   A_HMIDDLE, A_VMIDDLE, 1);
}

void
World::action(double frame_ratio)
{
  tux.action(frame_ratio);
  tux.check_bounds(level->back_scrolling, (bool)level->hor_autoscroll_speed);
  scrolling(frame_ratio);

  /* Handle bouncy distros: */
  unsigned int i = 0;
  for (i = 0; i < bouncy_distros.size(); i++)
    bouncy_distros[i]->action(frame_ratio);

  /* Handle broken bricks: */
  for (i = 0; i < broken_bricks.size(); i++)
    broken_bricks[i]->action(frame_ratio);

  // Handle all kinds of game objects
  for (i = 0; i < bouncy_bricks.size(); i++)
    bouncy_bricks[i]->action(frame_ratio);
  
  for (i = 0; i < floating_scores.size(); i++)
    floating_scores[i]->action(frame_ratio);

  for (i = 0; i < bullets.size(); ++i)
    bullets[i].action(frame_ratio);
  
  for (i = 0; i < upgrades.size(); i++)
    upgrades[i].action(frame_ratio);

  for (BadGuys::iterator it = bad_guys.begin(); it != bad_guys.end(); ++it)
    (*it)->action(frame_ratio);

  /* update particle systems */
  std::vector<ParticleSystem*>::iterator p;
  for(p = particle_systems.begin(); p != particle_systems.end(); ++p)
    {
      (*p)->simulate(frame_ratio);
    }

  /* Handle all possible collisions. */
  collision_handler();
  
  // Cleanup marked badguys
  for (it = bad_guys.begin(); it != bad_guys.end();
      /* ++i handled at end of the loop */) {
    if ((*it)->is_removable()) {
      delete *it;
      it =  bad_guys.erase(it);
    } else {
      ++it;
    }
  }
}

// the space that it takes for the screen to start scrolling, regarding
// screen bounds (in pixels)
#define X_SPACE  (screen->w - screen->w/6)  //(400-16)
// the time it takes to move the camera (in ms)
#define CHANGE_DIR_SCROLL_SPEED 2000

/* This functions takes cares of the scrolling */
void World::scrolling(double frame_ratio)
{
  if(level->hor_autoscroll_speed)
    {
    scroll_x += level->hor_autoscroll_speed * frame_ratio;
    return;
    }

  int tux_pos_x = (int)(tux.base.x + (tux.base.width/2));

  if (level->back_scrolling || debug_mode)
  {
    if(tux.old_dir != tux.dir && level->back_scrolling)
      scrolling_timer.start(CHANGE_DIR_SCROLL_SPEED);

    if(scrolling_timer.check())
    {
      float final_scroll_x;
      if (tux.physic.get_velocity_x() > 0)
        final_scroll_x = tux_pos_x - (screen->w - X_SPACE/myScale);
      else if (tux.physic.get_velocity_x() < 0)
        final_scroll_x = tux_pos_x - X_SPACE/myScale;
      else
      {
        if (tux.dir == RIGHT)
          final_scroll_x = tux_pos_x - (screen->w - X_SPACE/myScale);
        else if (tux.dir == LEFT && level->back_scrolling)
          final_scroll_x = tux_pos_x - X_SPACE/myScale;
      }

      scroll_x +=   (final_scroll_x - scroll_x)
                  / (frame_ratio * (CHANGE_DIR_SCROLL_SPEED / 100))
                  + (tux.physic.get_velocity_x() * frame_ratio + tux.physic.get_acceleration_x() * frame_ratio * frame_ratio);
      // std::cerr << tux_pos_x << " " << final_scroll_x << " " << scroll_x << std::endl;

    }
    else
    {
      if (tux.physic.get_velocity_x() > 0 && scroll_x < tux_pos_x - (screen->w - X_SPACE/myScale))
        scroll_x = tux_pos_x - (screen->w - X_SPACE/myScale);
      else if (tux.physic.get_velocity_x() < 0 && scroll_x > tux_pos_x - X_SPACE && level->back_scrolling)
        scroll_x = tux_pos_x - X_SPACE/myScale;
      else
      {
        if (tux.dir == RIGHT && scroll_x < tux_pos_x - (screen->w - X_SPACE/myScale))
            scroll_x = tux_pos_x - (screen->w - X_SPACE/myScale);
        else if (tux.dir == LEFT && scroll_x > tux_pos_x - X_SPACE/myScale && level->back_scrolling)
            scroll_x = tux_pos_x - X_SPACE/myScale;
      }
    }
  }

  else /*no debug*/
  {
    if (tux.physic.get_velocity_x() > 0 && scroll_x < tux_pos_x - (screen->w - X_SPACE/myScale))
      scroll_x = tux_pos_x - (screen->w - X_SPACE/myScale);
    else if (tux.physic.get_velocity_x() < 0 && scroll_x > tux_pos_x - X_SPACE/myScale && level->back_scrolling)
      scroll_x = tux_pos_x - X_SPACE/myScale;
    else
    {
      if (tux.dir == RIGHT && scroll_x < tux_pos_x - (screen->w - X_SPACE/myScale))
          scroll_x = tux_pos_x - (screen->w - X_SPACE/myScale);
      else if (tux.dir == LEFT && scroll_x > tux_pos_x - X_SPACE/myScale && level->back_scrolling)
          scroll_x = tux_pos_x - X_SPACE/myScale;
    }

  }

  // this code prevent the screen to scroll before the start or after the level's end
  if(scroll_x < 0)
    scroll_x = 0;
  if(scroll_x > level->width * 32/myScale - screen->w)
    scroll_x = level->width * 32/myScale - screen->w;
}

void
World::collision_handler()
{
  // CO_BULLET & CO_BADGUY check
  for(unsigned int i = 0; i < bullets.size(); ++i)
    {
      for (BadGuys::iterator j = bad_guys.begin(); j != bad_guys.end(); ++j)
        {
          if((*j)->dying != DYING_NOT)
            continue;
          
          if(rectcollision(bullets[i].base, (*j)->base))
            {
              // We have detected a collision and now call the
              // collision functions of the collided objects.
              // collide with bad_guy first, since bullet_collision will
              // delete the bullet
              (*j)->collision(0, CO_BULLET);
              bullets[i].collision(CO_BADGUY);
              break; // bullet is invalid now, so break
            }
        }
    }

  /* CO_BADGUY & CO_BADGUY check */
  for (BadGuys::iterator it = bad_guys.begin(); it != bad_guys.end(); ++it)
    {
      if((*it)->dying != DYING_NOT)
        continue;
      
      BadGuys::iterator j = it;
      ++j;
      for (; j != bad_guys.end(); ++j)
        {
          if(j == it || (*j)->dying != DYING_NOT)
            continue;

          if(rectcollision((*it)->base, (*j)->base))
            {
              // We have detected a collision and now call the
              // collision functions of the collided objects.
              (*j)->collision(*it, CO_BADGUY);
              (*it)->collision(*j, CO_BADGUY);
            }
        }
    }

  if(tux.dying != DYING_NOT) return;
    
  // CO_BADGUY & CO_PLAYER check 
  for ( it = bad_guys.begin(); it != bad_guys.end(); ++it)
    {
      if((*it)->dying != DYING_NOT)
        continue;
      
      if(rectcollision_offset((*it)->base, tux.base, 0, 0))
        {
          // We have detected a collision and now call the collision
          // functions of the collided objects.
          if (tux.previous_base.y < tux.base.y &&
              tux.previous_base.y + tux.previous_base.height 
              < (*it)->base.y + (*it)->base.height/2
              && !tux.invincible_timer.started())
            {
              (*it)->collision(&tux, CO_PLAYER, COLLISION_SQUISH);
            }
          else
            {
              tux.collision(*it, CO_BADGUY);
              (*it)->collision(&tux, CO_PLAYER, COLLISION_NORMAL);
            }
        }
    }

  // CO_UPGRADE & CO_PLAYER check
  for(i = 0; i < upgrades.size(); ++i)
    {
      if(rectcollision(upgrades[i].base, tux.base))
        {
          // We have detected a collision and now call the collision
          // functions of the collided objects.
          upgrades[i].collision(&tux, CO_PLAYER, COLLISION_NORMAL);
        }
    }
}

void
World::add_score(float x, float y, int s)
{
  player_status.score += s;

  FloatingScore* new_floating_score = new FloatingScore();
  new_floating_score->init(x,y,s);
  floating_scores.push_back(new_floating_score);
}

void
World::add_bouncy_distro(float x, float y)
{
  BouncyDistro* new_bouncy_distro = new BouncyDistro();
  new_bouncy_distro->init(x,y);
  bouncy_distros.push_back(new_bouncy_distro);
}

void
World::add_broken_brick(Tile* tile, float x, float y)
{
  add_broken_brick_piece(tile, x, y, -1.0/myScale, -4/myScale);
  add_broken_brick_piece(tile, x, y + (16/myScale), -1.5/myScale, -3/myScale);

  add_broken_brick_piece(tile, x + (16/myScale), y, 1/myScale, -4/myScale);
  add_broken_brick_piece(tile, x + (16/myScale), y + (16/myScale), 1.5/myScale, -3/myScale);
}

void
World::add_broken_brick_piece(Tile* tile, float x, float y, float xm, float ym)
{
  BrokenBrick* new_broken_brick = new BrokenBrick();
  new_broken_brick->init(tile, x, y, xm, ym);
  broken_bricks.push_back(new_broken_brick);
}

void
World::add_bouncy_brick(float x, float y)
{
  BouncyBrick* new_bouncy_brick = new BouncyBrick();
  new_bouncy_brick->init(x/myScale,y/myScale);
  bouncy_bricks.push_back(new_bouncy_brick);
}

BadGuy*
World::add_bad_guy(float x, float y, BadGuyKind kind, bool stay_on_platform)
{
  BadGuy* badguy = new BadGuy(x,y,kind, stay_on_platform);
  bad_guys.push_back(badguy);
  return badguy;
}

void
World::add_upgrade(float x, float y, Direction dir, UpgradeKind kind)
{
  Upgrade new_upgrade;
  new_upgrade.init(x,y,dir,kind);
  upgrades.push_back(new_upgrade);
}

void 
World::add_bullet(float x, float y, float xm, Direction dir)
{
  if(bullets.size() > MAX_BULLETS-1)
    return;

  Bullet new_bullet;
  new_bullet.init(x,y,xm,dir);
  bullets.push_back(new_bullet);
  
  play_sound(sounds[SND_SHOOT], SOUND_CENTER_SPEAKER);
}

void
World::play_music(int musictype)
{
  currentmusic = musictype;
  switch(currentmusic) {
    case HURRYUP_MUSIC:
      music_manager->play_music(get_level()->get_level_music_fast());
      break;
    case LEVEL_MUSIC:
      music_manager->play_music(get_level()->get_level_music());
      break;
    case HERRING_MUSIC:
      music_manager->play_music(herring_song);
      break;
    default:
      music_manager->halt_music();
      break;
  }
}

int
World::get_music_type()
{
  return currentmusic;
}

/* Break a brick: */
void
World::trybreakbrick(float x, float y, bool isSmall, Direction col_side)
{
  Level* plevel = get_level();
  
  Tile* tile = gettile(x, y);
  if (tile->brick)
    {
      if (tile->data > 0)
        {
          /* Get a distro from it: */
          add_bouncy_distro(((int)(x + 1) / (32/myScale)) * (32/myScale),
                                  (int)(y / (32/myScale)) * (32/myScale));

          // TODO: don't handle this in a global way but per-tile...
          if (!counting_distros)
            {
              counting_distros = true;
              distro_counter = 5;
            }
          else
            {
              distro_counter--;
            }

          if (distro_counter <= 0)
            {
              counting_distros = false;
              plevel->change(x, y, TM_IA, tile->next_tile);
            }

          play_sound(sounds[SND_DISTRO], SOUND_CENTER_SPEAKER);
          player_status.score = player_status.score + SCORE_DISTRO;
          player_status.distros++;
        }
      else if (!isSmall)
        {
          /* Get rid of it: */
          plevel->change(x, y, TM_IA, tile->next_tile);
          
          /* Replace it with broken bits: */
          add_broken_brick(tile, 
                                 ((int)(x + 1) / (32/myScale)) * (32/myScale),
                                 (int)(y / (32/myScale)) * (32/myScale));
          
          /* Get some score: */
          play_sound(sounds[SND_BRICK], SOUND_CENTER_SPEAKER);
          player_status.score = player_status.score + SCORE_BRICK;
        }
    }
  else if(tile->fullbox)
    tryemptybox(x, y, col_side);
}

/* Empty a box: */
void
World::tryemptybox(float x, float y, Direction col_side)
{
  Tile* tile = gettile(x,y);
  if (!tile->fullbox)
    return;

  // according to the collision side, set the upgrade direction
  if(col_side == LEFT)
    col_side = RIGHT;
  else
    col_side = LEFT;

  int posx = ((int)(x+1) / (32/myScale)) * (32/myScale);
  int posy = (int)(y/(32/myScale)) * (32/myScale) - (32/myScale);
  switch(tile->data)
    {
    case 1: // Box with a distro!
      add_bouncy_distro(posx, posy);
      play_sound(sounds[SND_DISTRO], SOUND_CENTER_SPEAKER);
      player_status.score = player_status.score + SCORE_DISTRO;
      player_status.distros++;
      break;

    case 2: // Add an upgrade!
      if (tux.size == SMALL)     /* Tux is small, add mints! */
        add_upgrade(posx, posy, col_side, UPGRADE_GROWUP);
      else     /* Tux is big, add an iceflower: */
        add_upgrade(posx, posy, col_side, UPGRADE_ICEFLOWER);
      play_sound(sounds[SND_UPGRADE], SOUND_CENTER_SPEAKER);
      break;

    case 3: // Add a golden herring
      add_upgrade(posx, posy, col_side, UPGRADE_HERRING);
      break;

    case 4: // Add a 1up extra
      add_upgrade(posx, posy, col_side, UPGRADE_1UP);
      break;
    default:
      break;
    }

  /* Empty the box: */
  level->change(x, y, TM_IA, tile->next_tile);
}

/* Try to grab a distro: */
void
World::trygrabdistro(float x, float y, int bounciness)
{
  Tile* tile = gettile(x, y);
  if (tile && tile->distro)
    {
      level->change(x, y, TM_IA, tile->next_tile);
      play_sound(sounds[SND_DISTRO], SOUND_CENTER_SPEAKER);

      if (bounciness == BOUNCE)
        {
          add_bouncy_distro(((int)(x + 1) / (32/myScale)) * (32/myScale),
                                  (int)(y / (32/myScale)) * (32/myScale));
        }

      player_status.score = player_status.score + SCORE_DISTRO;
      player_status.distros++;
    }
}

/* Try to bump a bad guy from below: */
void
World::trybumpbadguy(float x, float y)
{
  // Bad guys: 
  for (BadGuys::iterator it = bad_guys.begin(); it != bad_guys.end(); ++it)
    {
      if ((*it)->base.x >= x - (32/myScale) && (*it)->base.x <= x + (32/myScale) &&
          (*it)->base.y >= y - (16/myScale) && (*it)->base.y <= y + (16/myScale))
        {
          (*it)->collision(&tux, CO_PLAYER, COLLISION_BUMP);
        }
    }

  // Upgrades:
  for (unsigned int i = 0; i < upgrades.size(); i++)
    {
      if (upgrades[i].base.height == (32/myScale) &&
          upgrades[i].base.x >= x - (32/myScale) && upgrades[i].base.x <= x + (32/myScale) &&
          upgrades[i].base.y >= y - (16/myScale) && upgrades[i].base.y <= y + (16/myScale))
        {
          upgrades[i].collision(&tux, CO_PLAYER, COLLISION_BUMP);
        }
    }
}

/* EOF */

